import { BIP32Interface } from 'bip32';
import { TxOutput } from 'bitcoinjs-lib';
import { Taptree } from 'bitcoinjs-lib/src/types';
import { ECPairInterface } from 'ecpair';
import { OutputType } from './Enums';
export type Error = {
    message: string;
    code: string;
};
export type ScriptElement = Buffer | number | string;
export type TransactionOutput = {
    txHash: Buffer;
    vout: number;
    type: OutputType;
} & TxOutput;
export type RefundDetails = TransactionOutput & {
    keys: ECPairInterface | BIP32Interface;
    redeemScript?: Buffer;
    swapTree?: SwapTree;
    internalKey?: Buffer;
    cooperative?: boolean;
};
export type ClaimDetails = RefundDetails & {
    preimage: Buffer;
};
export type Tapleaf = {
    output: Buffer;
    version: number;
};
export type HashLeaf = {
    hash: Buffer;
};
export type HashBranch = {
    hash: Buffer;
    left: HashLeaf;
    right: HashLeaf;
};
export type SwapTree = {
    tree: Taptree;
    claimLeaf: Tapleaf;
    refundLeaf: Tapleaf;
};
export type LiquidSwapTree = SwapTree & {
    covenantClaimLeaf?: Tapleaf;
};
