"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getScriptIntrospectionValues = exports.getOutputValue = void 0;
const bitcoin_ops_1 = __importDefault(require("@boltz/bitcoin-ops"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const liquidjs_lib_1 = require("liquidjs-lib");
const init_1 = require("./init");
const getOutputValue = (output) => {
    return output.blindingPrivateKey &&
        output.rangeProof !== undefined &&
        output.rangeProof.length > 0
        ? Number(init_1.confidentialLiquid.unblindOutputWithKey(output, output.blindingPrivateKey).value)
        : liquidjs_lib_1.confidential.confidentialValueToSatoshi(output.value);
};
exports.getOutputValue = getOutputValue;
const getScriptIntrospectionWitnessScript = (outputScript) => outputScript.subarray(2, 40);
const getScriptIntrospectionValues = (outputScript) => {
    const dec = bitcoinjs_lib_1.script.decompile(outputScript);
    switch (dec[0]) {
        case bitcoin_ops_1.default.OP_1:
            return {
                version: 1,
                script: getScriptIntrospectionWitnessScript(outputScript),
            };
        case bitcoin_ops_1.default.OP_0:
            return {
                version: 0,
                script: getScriptIntrospectionWitnessScript(outputScript),
            };
        default:
            return {
                version: -1,
                script: bitcoinjs_lib_1.crypto.sha256(outputScript),
            };
    }
};
exports.getScriptIntrospectionValues = getScriptIntrospectionValues;
//# sourceMappingURL=Utils.js.map