"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.constructRefundTransaction = void 0;
const Utils_1 = require("../../Utils");
const Networks_1 = __importDefault(require("../consts/Networks"));
const Claim_1 = require("./Claim");
const dummyPreimage = (0, Utils_1.getHexBuffer)('0x00');
/**
 * Refund swaps
 *
 * @param utxos UTXOs that should be refunded
 * @param destinationScript the output script to which the funds should be sent
 * @param timeoutBlockHeight locktime of the transaction; only needed if the transaction is a refund
 * @param fee how many satoshis should be paid as fee
 * @param isRbf whether the transaction should signal full Replace-by-Fee
 * @param network network of the transaction
 * @param blindingKey blinding public key for the output; undefined if the output should not be blinded
 */
const constructRefundTransaction = (utxos, destinationScript, timeoutBlockHeight, fee, isRbf = true, network = Networks_1.default.liquidMainnet, blindingKey) => {
    const claimUtxos = utxos.map((utxo) => (Object.assign({ preimage: dummyPreimage }, utxo)));
    return (0, Claim_1.constructClaimTransaction)(claimUtxos, destinationScript, fee, isRbf, network, blindingKey, timeoutBlockHeight, true);
};
exports.constructRefundTransaction = constructRefundTransaction;
//# sourceMappingURL=Refund.js.map