"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Feature = void 0;
const bitcoin_ops_1 = __importDefault(require("@boltz/bitcoin-ops"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const bufferutils_1 = require("liquidjs-lib/src/bufferutils");
const Utils_1 = require("../../Utils");
const ReverseSwapTree_1 = __importDefault(require("../../swap/ReverseSwapTree"));
const TaprootUtils_1 = require("../../swap/TaprootUtils");
const TreeSort_1 = require("../../swap/TreeSort");
const Utils_2 = require("../Utils");
const Ops_1 = __importDefault(require("../consts/Ops"));
var Feature;
(function (Feature) {
    Feature[Feature["ClaimCovenant"] = 0] = "ClaimCovenant";
})(Feature || (exports.Feature = Feature = {}));
const claimCovenantOutputIndex = bitcoinjs_lib_1.script.number.encode(0);
const createClaimCovenantLeaf = (preimageHash, assetHash, outputScript, expectedAmount) => {
    const userOutput = (0, Utils_2.getScriptIntrospectionValues)(outputScript);
    const amountBuffer = Buffer.alloc(8);
    amountBuffer.writeBigUint64LE(BigInt(expectedAmount));
    return {
        version: TaprootUtils_1.leafVersionLiquid,
        output: bitcoinjs_lib_1.script.compile([
            bitcoin_ops_1.default.OP_SIZE,
            bitcoinjs_lib_1.script.number.encode(32),
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            bitcoin_ops_1.default.OP_HASH160,
            bitcoinjs_lib_1.crypto.ripemd160(preimageHash),
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            claimCovenantOutputIndex,
            Ops_1.default.OP_INSPECTOUTPUTSCRIPTPUBKEY,
            bitcoinjs_lib_1.script.number.encode(userOutput.version),
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            userOutput.script,
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            claimCovenantOutputIndex,
            Ops_1.default.OP_INSPECTOUTPUTASSET,
            bitcoin_ops_1.default.OP_1,
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            (0, bufferutils_1.reverseBuffer)((0, Utils_1.getHexBuffer)(assetHash)),
            bitcoin_ops_1.default.OP_EQUALVERIFY,
            claimCovenantOutputIndex,
            Ops_1.default.OP_INSPECTOUTPUTVALUE,
            bitcoin_ops_1.default.OP_DROP,
            amountBuffer,
            bitcoin_ops_1.default.OP_EQUAL,
        ]),
    };
};
const reverseSwapTree = (preimageHash, claimPublicKey, refundPublicKey, timeoutBlockHeight, features) => {
    const tree = (0, ReverseSwapTree_1.default)(true, preimageHash, claimPublicKey, refundPublicKey, timeoutBlockHeight);
    if (features === undefined) {
        return tree;
    }
    if (new Set(features.map((feature) => feature.type)).size !== features.length) {
        throw 'duplicate feature';
    }
    for (const feature of features) {
        switch (feature.type) {
            case Feature.ClaimCovenant:
                tree.covenantClaimLeaf = createClaimCovenantLeaf(preimageHash, feature.assetHash, feature.outputScript, feature.expectedAmount);
                break;
            default:
                throw `unknown feature: ${feature.type}`;
        }
    }
    return Object.assign(Object.assign({}, tree), { tree: (0, TreeSort_1.sortTree)((0, TreeSort_1.assignTreeProbabilities)(tree)) });
};
exports.default = reverseSwapTree;
//# sourceMappingURL=ReverseSwapTree.js.map