"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createControlBlock = exports.tweakMusig = exports.tapTweakHash = exports.tapBranchHash = exports.tapLeafHash = exports.hashForWitnessV1 = void 0;
exports.toHashTree = toHashTree;
const bip371_1 = require("bitcoinjs-lib/src/psbt/bip371");
const types_1 = require("bitcoinjs-lib/src/types");
const liquidjs_lib_1 = require("liquidjs-lib");
const bip341_1 = require("liquidjs-lib/src/bip341");
const crypto_1 = require("liquidjs-lib/src/crypto");
const Utils_1 = require("../../Utils");
const init_1 = require("../init");
const convertLeaf = (leaf) => ({
    version: leaf.version,
    scriptHex: (0, Utils_1.getHexString)(leaf.output),
});
const hashForWitnessV1 = (network, details, tx, index, leafHash, hashType = liquidjs_lib_1.Transaction.SIGHASH_DEFAULT) => {
    return tx.hashForWitnessV1(index, details.map((out) => out.script), details.map((out) => ({
        value: out.value,
        asset: out.asset,
    })), hashType, network.genesisBlockHash, leafHash);
};
exports.hashForWitnessV1 = hashForWitnessV1;
const tapLeafHash = (leaf) => (0, bip341_1.tapLeafHash)(convertLeaf(leaf));
exports.tapLeafHash = tapLeafHash;
const tapBranchHash = (a, b) => (0, crypto_1.taggedHash)('TapBranch/elements', Buffer.concat([a, b]));
exports.tapBranchHash = tapBranchHash;
const tapTweakHash = (publicKey, tweak) => (0, crypto_1.taggedHash)('TapTweak/elements', Buffer.concat([(0, bip371_1.toXOnly)(publicKey), tweak]));
exports.tapTweakHash = tapTweakHash;
function toHashTree(scriptTree) {
    if ((0, types_1.isTapleaf)(scriptTree)) {
        return { hash: (0, exports.tapLeafHash)(scriptTree) };
    }
    const hashes = [toHashTree(scriptTree[0]), toHashTree(scriptTree[1])];
    hashes.sort((a, b) => a.hash.compare(b.hash));
    const [left, right] = hashes;
    return {
        hash: (0, exports.tapBranchHash)(left.hash, right.hash),
        left,
        right,
    };
}
const tweakMusig = (musig, tree) => (0, bip371_1.toXOnly)(musig.tweakKey((0, exports.tapTweakHash)(musig.getAggregatedPublicKey(), toHashTree(tree).hash)));
exports.tweakMusig = tweakMusig;
const createControlBlock = (hashTree, leaf, internalKey) => {
    const path = (0, bip341_1.findScriptPath)(hashTree, (0, exports.tapLeafHash)(leaf));
    if (path === undefined || path.length === 0) {
        throw 'leaf not in tree';
    }
    const outputKey = init_1.secp.ecc.xOnlyPointAddTweak(internalKey, (0, exports.tapTweakHash)(internalKey, hashTree.hash));
    return Buffer.concat([Buffer.from([leaf.version | outputKey.parity]), internalKey].concat(path));
};
exports.createControlBlock = createControlBlock;
//# sourceMappingURL=TaprootUtils.js.map