"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.constructClaimTransaction = exports.validateInputs = exports.isRelevantTaprootOutput = void 0;
/**
 * This file is based on the repository github.com/submarineswaps/swaps-service created by Alex Bosworth
 */
const bitcoin_ops_1 = __importDefault(require("@boltz/bitcoin-ops"));
const bip65 = __importStar(require("bip65"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const bip341_1 = require("bitcoinjs-lib/src/payments/bip341");
const varuint = __importStar(require("varuint-bitcoin"));
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const SwapUtils_1 = require("./SwapUtils");
const TaprootUtils_1 = require("./TaprootUtils");
const dummyTaprootSignature = Buffer.alloc(64);
const isRelevantTaprootOutput = (utxo) => utxo.type === Enums_1.OutputType.Taproot && utxo.cooperative !== true;
exports.isRelevantTaprootOutput = isRelevantTaprootOutput;
const validateInputs = (utxos) => {
    if (utxos
        .filter((utxo) => utxo.type !== Enums_1.OutputType.Taproot)
        .some((utxo) => utxo.redeemScript === undefined)) {
        throw 'not all non Taproot inputs have a redeem script';
    }
    const relevantTaprootOutputs = utxos.filter(exports.isRelevantTaprootOutput);
    if (relevantTaprootOutputs.some((utxo) => utxo.swapTree === undefined)) {
        throw 'not all Taproot inputs have a swap tree';
    }
    if (relevantTaprootOutputs.some((utxo) => utxo.internalKey === undefined)) {
        throw 'not all Taproot inputs have an internal key';
    }
};
exports.validateInputs = validateInputs;
/**
 * Claim swaps
 *
 * @param utxos UTXOs that should be claimed or refunded
 * @param destinationScript the output script to which the funds should be sent
 * @param fee how many satoshis should be paid as fee
 * @param isRbf whether the transaction should signal full Replace-by-Fee
 * @param timeoutBlockHeight locktime of the transaction; only needed if the transaction is a refund
 * @param isRefund whether the transaction is a refund or claim
 */
const constructClaimTransaction = (utxos, destinationScript, fee, isRbf = true, timeoutBlockHeight, isRefund = false) => {
    (0, exports.validateInputs)(utxos);
    const tx = new bitcoinjs_lib_1.Transaction();
    // Refund transactions are just like claim ones and therefore this method
    // is also used for refunds. The locktime of refund transactions has to be
    // after the timelock of the UTXO is expired
    if (timeoutBlockHeight) {
        tx.locktime = bip65.encode({ blocks: timeoutBlockHeight });
    }
    // The sum of the values of all UTXOs that should be claimed or refunded
    let utxoValueSum = BigInt(0);
    utxos.forEach((utxo) => {
        utxoValueSum += BigInt(utxo.value);
        // Add the swap as input to the transaction
        // RBF reference: https://github.com/bitcoin/bips/blob/master/bip-0125.mediawiki#summary
        tx.addInput(utxo.txHash, utxo.vout, isRbf ? 0xfffffffd : 0xffffffff);
    });
    // Send the sum of the UTXOs minus the estimated fee to the destination address
    tx.addOutput(destinationScript, Number(utxoValueSum - BigInt(fee)));
    utxos.forEach((utxo, index) => {
        switch (utxo.type) {
            // Construct and sign the input scripts for P2SH inputs
            case Enums_1.OutputType.Legacy: {
                const sigHash = tx.hashForSignature(index, utxo.redeemScript, bitcoinjs_lib_1.Transaction.SIGHASH_ALL);
                const signature = utxo.keys.sign(sigHash);
                const inputScript = [
                    (0, SwapUtils_1.encodeSignature)(bitcoinjs_lib_1.Transaction.SIGHASH_ALL, Buffer.from(signature)),
                    utxo.preimage,
                    bitcoin_ops_1.default.OP_PUSHDATA1,
                    utxo.redeemScript,
                ];
                tx.setInputScript(index, (0, SwapUtils_1.scriptBuffersToScript)(inputScript));
                break;
            }
            // Construct the nested redeem script for nested SegWit inputs
            case Enums_1.OutputType.Compatibility: {
                const nestedScript = [
                    (0, Utils_1.getHexString)(Buffer.from(varuint.encode(bitcoin_ops_1.default.OP_0).buffer)),
                    bitcoinjs_lib_1.crypto.sha256(utxo.redeemScript),
                ];
                const nested = (0, SwapUtils_1.scriptBuffersToScript)(nestedScript);
                tx.setInputScript(index, (0, SwapUtils_1.scriptBuffersToScript)([nested]));
                break;
            }
        }
        // Construct and sign the witness for (nested) SegWit inputs
        // When the Taproot output is spent cooperatively, we leave it empty
        if (utxo.type === Enums_1.OutputType.Taproot) {
            if (utxo.cooperative !== true) {
                const tapLeaf = isRefund
                    ? utxo.swapTree.refundLeaf
                    : utxo.swapTree.claimLeaf;
                const sigHash = (0, TaprootUtils_1.hashForWitnessV1)(utxos, tx, index, (0, bip341_1.tapleafHash)(tapLeaf), bitcoinjs_lib_1.Transaction.SIGHASH_DEFAULT);
                const signature = Buffer.from(utxo.keys.signSchnorr(sigHash));
                const witness = isRefund ? [signature] : [signature, utxo.preimage];
                tx.setWitness(index, witness.concat([
                    tapLeaf.output,
                    (0, TaprootUtils_1.createControlBlock)((0, bip341_1.toHashTree)(utxo.swapTree.tree), tapLeaf, utxo.internalKey),
                ]));
            }
            else {
                // Stub the signature to allow for accurate fee estimations
                tx.setWitness(index, [dummyTaprootSignature]);
            }
        }
        else if (utxo.type === Enums_1.OutputType.Bech32 ||
            utxo.type === Enums_1.OutputType.Compatibility) {
            const sigHash = tx.hashForWitnessV0(index, utxo.redeemScript, utxo.value, bitcoinjs_lib_1.Transaction.SIGHASH_ALL);
            const signature = bitcoinjs_lib_1.script.signature.encode(Buffer.from(utxo.keys.sign(sigHash)), bitcoinjs_lib_1.Transaction.SIGHASH_ALL);
            tx.setWitness(index, [signature, utxo.preimage, utxo.redeemScript]);
        }
    });
    return tx;
};
exports.constructClaimTransaction = constructClaimTransaction;
//# sourceMappingURL=Claim.js.map