"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractRefundPublicKeyFromReverseSwapTree = exports.extractClaimPublicKeyFromReverseSwapTree = void 0;
const bitcoin_ops_1 = __importDefault(require("@boltz/bitcoin-ops"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const bip371_1 = require("bitcoinjs-lib/src/psbt/bip371");
const SwapTree_1 = require("./SwapTree");
const TaprootUtils_1 = require("./TaprootUtils");
const extractClaimPublicKeyFromReverseSwapTree = (swapTree) => bitcoinjs_lib_1.script.decompile(swapTree.claimLeaf.output)[6];
exports.extractClaimPublicKeyFromReverseSwapTree = extractClaimPublicKeyFromReverseSwapTree;
const extractRefundPublicKeyFromReverseSwapTree = (swapTree) => (0, SwapTree_1.extractRefundPublicKeyFromSwapTree)(swapTree);
exports.extractRefundPublicKeyFromReverseSwapTree = extractRefundPublicKeyFromReverseSwapTree;
const reverseSwapTree = (isLiquid, preimageHash, claimPublicKey, refundPublicKey, timeoutBlockHeight) => {
    const claimLeaf = (0, TaprootUtils_1.createLeaf)(isLiquid, [
        bitcoin_ops_1.default.OP_SIZE,
        bitcoinjs_lib_1.script.number.encode(32),
        bitcoin_ops_1.default.OP_EQUALVERIFY,
        bitcoin_ops_1.default.OP_HASH160,
        bitcoinjs_lib_1.crypto.ripemd160(preimageHash),
        bitcoin_ops_1.default.OP_EQUALVERIFY,
        (0, bip371_1.toXOnly)(claimPublicKey),
        bitcoin_ops_1.default.OP_CHECKSIG,
    ]);
    const refundLeaf = (0, SwapTree_1.createRefundLeaf)(isLiquid, refundPublicKey, timeoutBlockHeight);
    return {
        claimLeaf,
        refundLeaf,
        tree: (0, TaprootUtils_1.swapLeafsToTree)(claimLeaf, refundLeaf),
    };
};
exports.default = reverseSwapTree;
//# sourceMappingURL=ReverseSwapTree.js.map