"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectSwap = void 0;
const Enums_1 = require("../consts/Enums");
const Scripts_1 = require("./Scripts");
/**
 * Detects a swap output with the matching redeem script or tweaked key in a transaction
 */
const detectSwap = (redeemScriptOrTweakedKey, transaction) => {
    const scripts = [
        [Enums_1.OutputType.Legacy, (0, Scripts_1.p2shOutput)(redeemScriptOrTweakedKey)],
        [Enums_1.OutputType.Compatibility, (0, Scripts_1.p2shP2wshOutput)(redeemScriptOrTweakedKey)],
        [Enums_1.OutputType.Bech32, (0, Scripts_1.p2wshOutput)(redeemScriptOrTweakedKey)],
        [Enums_1.OutputType.Taproot, (0, Scripts_1.p2trOutput)(redeemScriptOrTweakedKey)],
    ];
    for (const [vout, output] of transaction.outs.entries()) {
        const scriptMatch = scripts.find(([, script]) => script.equals(output.script));
        if (scriptMatch) {
            return Object.assign({ vout, type: scriptMatch[0] }, output);
        }
    }
    return;
};
exports.detectSwap = detectSwap;
//# sourceMappingURL=SwapDetector.js.map