"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractRefundPublicKeyFromSwapTree = exports.extractClaimPublicKeyFromSwapTree = exports.createRefundLeaf = void 0;
const bitcoin_ops_1 = __importDefault(require("@boltz/bitcoin-ops"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const bip371_1 = require("bitcoinjs-lib/src/psbt/bip371");
const SwapUtils_1 = require("./SwapUtils");
const TaprootUtils_1 = require("./TaprootUtils");
const createRefundLeaf = (isLiquid, refundPublicKey, timeoutBlockHeight) => (0, TaprootUtils_1.createLeaf)(isLiquid, [
    (0, bip371_1.toXOnly)(refundPublicKey),
    bitcoin_ops_1.default.OP_CHECKSIGVERIFY,
    (0, SwapUtils_1.encodeCltv)(timeoutBlockHeight),
    bitcoin_ops_1.default.OP_CHECKLOCKTIMEVERIFY,
]);
exports.createRefundLeaf = createRefundLeaf;
const extractClaimPublicKeyFromSwapTree = (swapTree) => bitcoinjs_lib_1.script.decompile(swapTree.claimLeaf.output)[3];
exports.extractClaimPublicKeyFromSwapTree = extractClaimPublicKeyFromSwapTree;
const extractRefundPublicKeyFromSwapTree = (swapTree) => bitcoinjs_lib_1.script.decompile(swapTree.refundLeaf.output)[0];
exports.extractRefundPublicKeyFromSwapTree = extractRefundPublicKeyFromSwapTree;
const swapTree = (isLiquid, preimageHash, claimPublicKey, refundPublicKey, timeoutBlockHeight) => {
    const claimLeaf = (0, TaprootUtils_1.createLeaf)(isLiquid, [
        bitcoin_ops_1.default.OP_HASH160,
        bitcoinjs_lib_1.crypto.ripemd160(preimageHash),
        bitcoin_ops_1.default.OP_EQUALVERIFY,
        (0, bip371_1.toXOnly)(claimPublicKey),
        bitcoin_ops_1.default.OP_CHECKSIG,
    ]);
    const refundLeaf = (0, exports.createRefundLeaf)(isLiquid, refundPublicKey, timeoutBlockHeight);
    return {
        claimLeaf,
        refundLeaf,
        tree: (0, TaprootUtils_1.swapLeafsToTree)(claimLeaf, refundLeaf),
    };
};
exports.default = swapTree;
//# sourceMappingURL=SwapTree.js.map