"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeSwapTree = exports.serializeSwapTree = void 0;
const Utils_1 = require("../Utils");
const TreeSort_1 = require("./TreeSort");
const serializeLeaf = (leaf) => ({
    version: leaf.version,
    output: (0, Utils_1.getHexString)(leaf.output),
});
const deserializeLeaf = (leaf) => ({
    version: leaf.version,
    output: (0, Utils_1.getHexBuffer)(leaf.output),
});
const serializeSwapTree = (tree) => {
    const res = {
        claimLeaf: serializeLeaf(tree.claimLeaf),
        refundLeaf: serializeLeaf(tree.refundLeaf),
    };
    if (tree.covenantClaimLeaf !== undefined) {
        res.covenantClaimLeaf = serializeLeaf(tree.covenantClaimLeaf);
    }
    return res;
};
exports.serializeSwapTree = serializeSwapTree;
const deserializeSwapTree = (tree) => {
    const parsed = typeof tree === 'string' ? JSON.parse(tree) : tree;
    const res = {
        claimLeaf: deserializeLeaf(parsed.claimLeaf),
        refundLeaf: deserializeLeaf(parsed.refundLeaf),
        covenantClaimLeaf: parsed.covenantClaimLeaf !== undefined
            ? deserializeLeaf(parsed.covenantClaimLeaf)
            : undefined,
    };
    return Object.assign(Object.assign({}, res), { tree: (0, TreeSort_1.sortTree)((0, TreeSort_1.assignTreeProbabilities)(res)) });
};
exports.deserializeSwapTree = deserializeSwapTree;
//# sourceMappingURL=SwapTreeSerializer.js.map