"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tweakMusig = exports.hashForWitnessV1 = exports.createLeaf = exports.createControlBlock = exports.swapLeafsToTree = exports.leafVersionLiquid = void 0;
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const bip341_1 = require("bitcoinjs-lib/src/payments/bip341");
const bip371_1 = require("bitcoinjs-lib/src/psbt/bip371");
const SwapUtils_1 = require("./SwapUtils");
exports.leafVersionLiquid = 196;
const swapLeafsToTree = (claimLeaf, refundLeaf) => [claimLeaf, refundLeaf];
exports.swapLeafsToTree = swapLeafsToTree;
const createControlBlock = (hashTree, leaf, internalKey) => {
    const path = (0, bip341_1.findScriptPath)(hashTree, (0, bip341_1.tapleafHash)(leaf));
    if (path === undefined) {
        throw 'leaf not in tree';
    }
    const outputKey = (0, bip341_1.tweakKey)(internalKey, hashTree.hash);
    return Buffer.concat([Buffer.from([leaf.version | outputKey.parity]), internalKey].concat(path));
};
exports.createControlBlock = createControlBlock;
const createLeaf = (isLiquid, script) => ({
    version: isLiquid ? exports.leafVersionLiquid : bip341_1.LEAF_VERSION_TAPSCRIPT,
    output: (0, SwapUtils_1.toPushdataScript)(script),
});
exports.createLeaf = createLeaf;
const hashForWitnessV1 = (details, tx, index, leafHash, hashType = bitcoinjs_lib_1.Transaction.SIGHASH_DEFAULT) => {
    return tx.hashForWitnessV1(index, details.map((out) => out.script), details.map((out) => out.value), hashType, leafHash);
};
exports.hashForWitnessV1 = hashForWitnessV1;
const tweakMusig = (musig, tree) => (0, bip371_1.toXOnly)(musig.tweakKey((0, bip341_1.tapTweakHash)(musig.getAggregatedPublicKey(), (0, bip341_1.toHashTree)(tree).hash)));
exports.tweakMusig = tweakMusig;
//# sourceMappingURL=TaprootUtils.js.map